<?php

namespace Teknisa\DF\Modules\Entrada\Services;

use Doctrine\ORM\EntityManager;

//OLD Entities
use Geral\Entities\Itemproduto;
use Geral\Entities\Itemautf;
use Geral\Entities\Produto;
use Geral\Entities\Encargo;
use Geral\Entities\Inscrestad;
use Fiscal\Entities\Notafiscal;
use Fiscal\Entities\IntgNotafiscal;
use Fiscal\Entities\Trabitproimpos;
use Fiscal\Entities\Trabimposnftot;
use Fiscal\Entities\Imposnftot;
use Fiscal\Entities\Itemproimpos;
use Estoque\Entities\Lanctoestoq;
use Suprimentos\Entities\Encargoaf;
use Suprimentos\Entities\Impitmaf;
use Suprimentos\Entities\Nfrelacaf;
use Indeterminado\Entities\Prodfornxml;

use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\NewCode;
use Teknisa\DF\Services\ValidacoesEstoqueService;
use Teknisa\DF\Services\VencimentoService;
use Teknisa\DF\Factories\NotafiscalLancamentoFactory;
use Teknisa\DF\Modules\Entrada\Factories\Dfe11000NfEntradaFactory;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\Remote\FileUploader\Server;

class Dfe11000NfEntradaService {
    
    const ALTERACAO = 'A';

    protected $entityManager;
    protected $novoCodigo;
    protected $lancamentoService;
    protected $validacoesEstoqueService;
    protected $environment;

    private $notafiscal;
    private $filial;
    private $lancamento;
    private $organizacao;
    private $lancamentoFactory;
    private $lancamentoEntradaFactory;
    private $vencimentoService;
    private $fileService;
    private $fileUploader;

    //TODO Remover parametros não usados
    public function __construct(EntityManager $entityManager, NewCode $novoCodigo, $environment, $lancamentoService = null, $validacoesEstoqueService = null, $fileService = null, $vencimentoService = null, Server $fileUploader = null, $notafiscal = null) {
        $this->novoCodigo = $novoCodigo;
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->lancamentoService = $lancamentoService;
        $this->validacoesEstoqueService = $validacoesEstoqueService;
        
        if ($notafiscal instanceof Notafiscal) {
            $this->setVariaveisNota($notafiscal);
        }

        //TODO nao instanciar a classe
        $this->lancamentoFactory        = new NotafiscalLancamentoFactory($entityManager);
        $this->lancamentoEntradaFactory = new Dfe11000NfEntradaFactory($entityManager);
        $this->vencimentoService        = $vencimentoService;
        $this->fileService = $fileService;
        $this->fileUploader = $fileUploader;
    }

    public static function instanceService(
        EntityManager $entityManager, NewCode $novoCodigo, $environment, $filial, $lancamento, $lancamentoService = null, $validacoesEstoqueService = null, $organizacao = null
    ) {
        if (!$organizacao)
            $organizacao = $environment->getNrOrgTrab();

        $notafiscal = self::getNotafiscalEntity($filial, $lancamento, $organizacao);
        return new self($entityManager, $novoCodigo, $environment, $lancamentoService, $validacoesEstoqueService, $notafiscal);
    }

    public function setNotafiscalEntity($filial, $lancamento, $organizacao = null) {
        if (!$organizacao)
            $organizacao = $this->environment->getNrOrgTrab();
            

        $notafiscal = self::getNotafiscalEntity($filial, $lancamento, $organizacao);
        $this->setVariaveisNota($notafiscal);
    }

    private static function getNotafiscalEntity($filial, $lancamento, $organizacao) {

        $notafiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);

        if (!is_object($notafiscal))
            throw new \Exception('Operação Bloqueada! Nota fiscal não encontrada.');

        return $notafiscal;
    }

    private function setVariaveisNota(Notafiscal $notafiscal) {
        $this->notafiscal = $notafiscal;
        $this->filial = $notafiscal->getCdfilial();
        $this->lancamento = $notafiscal->getNrlanctonf();
        $this->organizacao = ($notafiscal->getNrorg()) ? $notafiscal->getNrorg() : $this->environment->getNrOrgTrab();
    }

    // Formata a data de forma que possa ser inserida no banco
    public static function formataData($data) {
        if (!$data || !is_string($data))
            return $data;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);

        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);
        return $data;
    }

    /**
     * Atualiza status da AF
     */
    public function atualizaStatusAF($filial, $lancamento, $organizacao, $status) {
        try {
            $itensNota = Itemproduto::findByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);

            if (is_array($itensNota)) {
                foreach ($itensNota as $item) {
                    $this->lancamentoEntradaFactory->atualizaStatusAF($item, $status);
                }
            }
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    

    /***************************** Nota Fiscal - Delete *********************************************/

     public function deleteNotaFiscalEntrada() {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
                
            $filial = $this->filial;
            $lancamento = $this->lancamento;
            
            $dadosNfIntg = $this->lancamentoEntradaFactory->getDadosNfIntg($filial, $lancamento);
            
            $anexos = $this->lancamentoEntradaFactory->getAnexoNfe($filial, $lancamento);
            
            $organizacao = $this->organizacao;
            $lanctoestoq = Lanctoestoq::findOneByCdfilialAndNrlanctonf($filial, $lancamento);
            
    
            $this->verificaExistenciaOcorrenciaAtendimento($filial, $lancamento, $organizacao);
            $this->notaEmitidaAprovada($this->notafiscal);
    
            $this->atualizaStatusAF($filial, $lancamento, $organizacao, 'E');

            if ($this->notafiscal->getNracessonfe()) {
                $this->atualizaStatusXML($this->notafiscal);
                $this->lancamentoService->atualizaNotaDestinada('', '', $this->notafiscal->getNracessonfe());
            }
            
            $this->deletaRelacNotaOrigem($filial, $lancamento);
            $this->deletaImpostosTemporarios($filial, $lancamento, $organizacao);
            $this->lancamentoEntradaFactory->deleteItemnotafisc($filial, $lancamento);
            $this->lancamentoEntradaFactory->deleteAnexoNF($filial, $lancamento);
            $this->lancamentoFactory->procedureDeletaNf($filial, $lancamento);
            
            if($anexos){
                foreach($anexos as $anexo){
                    $anexo = explode(">", $anexo)[1];
                    $this->deletarAnexoFileServer($anexo);
                }
            }
            
            
            if($dadosNfIntg["NRACESSONFE"]) {
                $chave = $dadosNfIntg["NRACESSONFE"];
                $sequencial = $this->lancamentoEntradaFactory->getSequencialHistoricoXML($chave)[0]["NRSEQHISTOR"];
                if($sequencial === null) $sequencial = 1;
                else $sequencial += 1;
                $sequencial = str_pad($sequencial, 20, "0", STR_PAD_LEFT);
                $dsOperacao = 'Exclusão de Nota a partir de arquivo XML';
                $this->lancamentoEntradaFactory->insereHistoricoXML($chave, $sequencial, date('d/m/Y H:i:s'), '2', 'N', '', 
                                $dsOperacao, $filial, $lancamento, $this->environment->getCdOperador(), $this->environment->getNrOrgTrab());
            }

            if (is_object($lanctoestoq)) {
                $lancamentoEstoque = $lanctoestoq->getNrlancestq();
                $this->validacoesEstoqueService->setLanctoestoq($lanctoestoq);
                $this->validacoesEstoqueService->atualizaEstoque($filial, $lancamentoEstoque);
            }
            
            
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }
    
    public function validaParamProduto($row){
        try {
            return $this->lancamentoEntradaFactory->validaParamProduto($row);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function deletaRelacNotaOrigem($filial, $nrLanctoNF){
        $this->lancamentoFactory->deletaRelacNotaOrigem($filial, $nrLanctoNF);
    }

    private function verificaExistenciaOcorrenciaAtendimento($filial, $lancamento, $organizacao) {
        try {
            $movocorr = \Qualificacao\Entities\Movocorr::findOneByNrlanctonfAndCdfilialnfAndNrorg($filial, $lancamento, $organizacao);

            if (is_object($movocorr))
                throw new \Exception('Nota fiscal não pode ser excluída! Existe ocorrência de atendimento para esta nota. ' . $movocorr->getNrocorrmov());
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function notaEmitidaAprovada(Notafiscal $notafiscal) {
        try {
            if ($notafiscal->getIdstatus() === 'A' || $notafiscal->getIdstatus() === 'E')
                throw new \Exception('Operação Bloqueada! Nota fiscal emitida ou aprovada.');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function atualizaStatusXML(Notafiscal $notafiscal) {
        try {
            /** @var $xml IntgNotafiscal */
            $xml = IntgNotafiscal::findOneByNracessonfe($notafiscal->getNracessonfe());

            if (is_object($xml)) {
                $xml->setIdstatusnfe('N');
                $xml->setIdimportada('N');
                $xml->setDtutilizaxml(null);
                $xml->setCdfilial(null);
                $xml->setNrlanctonf(null);
                $xml->persist();
            }
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function deletaImpostosTemporarios($filial, $lancamento, $organizacao) {
        $this->deletaImpostosTemporariosItem($filial, $lancamento, $organizacao);
        $this->deletaImpostosTemporariosNota($filial, $lancamento, $organizacao);
    }

    private function deletaImpostosTemporariosItem($filial, $lancamento, $organizacao) {
        try {
            /** @var $trabitproimpos Trabitproimpos */
            $trabitproimpos = Trabitproimpos::findByCdfilimpoitAndNrlancimpoitAndNrorg($filial, $lancamento, $organizacao);
            if (is_array($trabitproimpos)) {
                foreach ($trabitproimpos as $trabImposto) {
                    $trabImposto->delete();
                }
            }
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function deletaImpostosTemporariosNota($filial, $lancamento, $organizacao) {
        try {
            /** @var $trabimposnftot Trabimposnftot */
            $trabimposnftot = Trabimposnftot::findByCdfilimptotAndNrlancimptotAndNrorg($filial, $lancamento, $organizacao);

            if (is_array($trabimposnftot)) {
                foreach ($trabimposnftot as $imposto){
                    $imposto->delete();
               }
           }
           $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /***************************** / Nota Fiscal - Delete ********************************************/

    /***************************** Nota Fiscal - Importação AF ***************************************/
    public function realizaImportacaoAf($cabecalho, array $afSelecionadas) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $filial = $cabecalho['CDFILIAL'];
            $lancamento = $cabecalho['NRLANCTONF'];
            $dtrefind =  $cabecalho['DTEMISSAO'];

            $dtrefind =  \DateTime::createFromFormat("d/m/Y", $dtrefind);
            $dtrefind = date_sub($dtrefind, date_interval_create_from_date_string('1 days'))->format('d/m/Y');
            
            $operador = $this->environment->getCdOperador();
            $organizacao = $this->environment->getNrOrgTrab();
            $nrseqitprod = 0;
            $arrayEncargos = array();
            $arrayImpostosItem = array();
            $itensAf = array();
            //Loop nas AF's selecionados pelo usuário
            foreach ($afSelecionadas as $af) {
                $filialLanc = $af['CDFILLANC'];
                $filialDestAf = $af['CDFILDEST'];
                $nrAutoForn = $af['NRAUTOFORN'];
                $dtProgentAf = $af['DTPROGENTAF'];
                $dtAutoForn = $af['DTAUTOFORN'];
                $vrTotalAf = $af['VRTOTALAF'];
                $cdindice =  $af['CDINDICE'];
                $cdtpindice = $af['CDTPINDICE'];
                $idpervalor = $af['IDPERVALOR'];
                
                $this->buscaItensAf($itensAf, $filialDestAf, $nrAutoForn, $cdindice);
                $this->buscaEncargosAf($arrayEncargos, $filialDestAf, $filialLanc, $nrAutoForn, $dtProgentAf, $organizacao);
            }

            $this->salvaItensAf(
                $cabecalho, $itensAf, $filial, $lancamento, $filialLanc, $filialDestAf, $nrAutoForn,
                $dtProgentAf, $nrseqitprod, $organizacao, $operador, $arrayImpostosItem, $dtrefind, $cdindice,
                $cdtpindice, $idpervalor
            );

            $this->salvaImpostosAf(
                $arrayImpostosItem, $filial, $lancamento, $filialDestAf,
                $filialLanc, $nrAutoForn, $dtProgentAf, $operador, $organizacao
            );
            
            $arrayEncargos = $this->salvaEncargosAf(
                $arrayEncargos, $filial, $lancamento, $organizacao, $operador
            );

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    /**
     * Salva os itens da AF na nota que esta sendo cadastrada.
     */
    private function salvaItensAf(
        $cabecalho, $itensAf, $filial, $lancamento, $filialLanc, $filialDestAf,
        $nrAutoForn, $dtProgentAf, &$nrseqitprod, $organizacao, $operador, &$arrayImpostosItem, $dtrefind ,$cdindice,
        $cdtpindice, $idpervalor
    ) {
        try {
            if(is_array($itensAf)) {
                foreach($itensAf as $item) {
                    $produto = Produto::findOneByCdprodutoAndNrorg($item['CDPRODUTO'], $organizacao);

                    $nrseqitprod = str_pad(++$nrseqitprod, 4, '0', STR_PAD_LEFT);
                    $cdproduto = $item['CDPRODUTO'];
                    $cdmarca = $item['CDMARCA'];
                    // $dtprogentaf = $item['DTPROGENTAF']->format('d/m/Y');
                    $qtprogentaf = floatval($item['QTPROGENTAF']);
                    $qtitemprod = $qtprogentaf - floatval($item['QTEFETENTRAF']);
                    $idbaixitmaf = $item['IDBAIXITMAF'];

                    $itemBaixado = (($idbaixitmaf !== '2' && $idbaixitmaf !== '3') || !$idbaixitmaf);
                    $validacaoGravaItem = ($qtitemprod > 0);
                        
                    if ($validacaoGravaItem || $itemBaixado) {
                        $cdfilimovi = $this->notafiscal->getCdfilirece();

                        $cdprodemis = $cdproduto;
                        $dsprodemis = $produto->getNmproduto();

                        $vrpreuniaf = $item['VRPREUNIAF'];
                        $vrbrutitem = $vrpreuniaf;
                        $vrunititem = $vrpreuniaf;
                        $vrdescitem = floatval($item['VRDESCITMAF']);
                        $vrtotaitem = $vrunititem * $qtitemprod - (($vrunititem * $qtitemprod) * ($item['VRPERDESAF'] / 100));

                      if($cdindice != null){

                            $vrIndice = $this->lancamentoEntradaFactory->getValorIndicePorDataEmissao($cdindice, $cdtpindice, $idpervalor, $dtrefind, $this->environment->getNrOrgTrab());
                            if ($vrIndice != null){
                              $vrdescitem = ($vrunititem * $qtitemprod) * ($item['VRPERDESAF'] / 100);
                              $vrpreuniaf = $this->converterValorMoeda($vrIndice, $vrpreuniaf);
                              $vrbrutitem = $this->converterValorMoeda($vrIndice, $vrbrutitem);
                              $vrunititem = $this->converterValorMoeda($vrIndice, $vrunititem);
                              $vrdescitem = $this->converterValorMoeda($vrIndice, $vrdescitem);
                              $vrtotaitem = $this->converterValorMoeda($vrIndice, $vrtotaitem);
                            }else{
                                throw new \Exception('Não Foi possível realizar a conversão de moeda! Nao há valor do índice '.$vrIndice.' para a data '.$dtrefind); 
                            }
                        }
  
                        $this->lancamentoFactory->novoItemproduto(
                            $filial, $lancamento, $nrseqitprod, $cdproduto, $cdprodemis,
                            $dsprodemis, $qtitemprod, $vrbrutitem, $vrunititem, $vrdescitem,
                            $vrtotaitem, $organizacao, $operador, $nrAutoForn, $cdfilimovi,
                            $cdmarca, $dtProgentAf
                        );
                        
                        $this->entityManager->flush();
                            
                        $this->salvaImpostosItemAf($cabecalho, array(
                            'CDFILIAL' => $filial,
                            'NRLANCTONF' => $lancamento,
                            'NRSEQITPROD' => $nrseqitprod,
                            'CDARVPROD' => $produto->getCdarvprod(),
                            'VRTOTAITEM' => $vrtotaitem,
                            'NRORG' => $organizacao,
                            '__generate_taxes' => true
                        ), $arrayImpostosItem);

                        $this->entityManager->flush();

                        $QTRESTANTE = $qtitemprod;
        
                        foreach ($item['PARENT_INFO'] as $parent) {
                            $QTNFEFETAF = ($QTRESTANTE > $parent['QTPROGENTAF'])? $parent['QTPROGENTAF'] : $QTRESTANTE;
                            $QTRESTANTE -= $QTNFEFETAF;

                            if ($QTNFEFETAF > 0) {
                                //Cria o relacionamento entre os produtos da AF e a nota fiscal.

                                $this->novoRelacionamentoAfNf(
                                    $filial, $lancamento, $filialDestAf, $filialLanc, $parent['NRAUTOFORN'],
                                    $cdmarca, $cdproduto, $parent['DTPROGENTAF'], $parent['QTPROGENTAF'], $vrpreuniaf,
                                    $QTNFEFETAF, $organizacao, $operador
                                );
                            }
                        }
                    }
                }                
/*                if($vrTotaldeItens != $vrNotaFiscal) {
                    throw new \Exception('Valor da Nota Fiscal difere do valor total de itens'); 
                    
                }*/
                

                //Chama procedure NF_BAIXA_AF para dar baixa do item na AF
                $this->entityManager->flush();
                $this->atualizaStatusAF($filial, $lancamento, $organizacao, 'I');
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Cria o relacionamento entre os produtos da AF e a nota fiscal.
     */
    private function novoRelacionamentoAfNf(
        $filial, $lancamento, $filialDestAf, $filialLanc, $nrAutoForn,
        $cdmarca, $cdproduto, $dtProgentAf, $qtprogentaf, $vrpreuniaf,
        $qtnfefetaf, $organizacao, $operador
    ) {
        $this->lancamentoEntradaFactory->novoRelacionamentoAfNf(
            $filial, $lancamento, $filialDestAf, $filialLanc, $nrAutoForn,
            $cdmarca, $cdproduto, $dtProgentAf, $qtprogentaf, $vrpreuniaf,
            $qtnfefetaf, $organizacao, $operador
        );
    }

    /**
     * Busca os impostos da AF.
     */
    private function buscaImpostosAf($filialDestAf, $filialLanc, $nrAutoForn, $dtProgentAf, $cdproduto, $organizacao) {
        $dtProgentAfFormat = self::formataData($dtProgentAf);
        $impitmaf = Impitmaf::findByCdfildestAndCdfillancAndNrautofornAndDtprogentafAndCdprodutoAndNrorg(
            $filialDestAf, $filialLanc, $nrAutoForn, $dtProgentAfFormat, $cdproduto, $organizacao
        );
        return $impitmaf;
    }

    /**
     * Salva os impostos na nota com base na soma dos impostos dos itens da AF.
     */
    private function salvaImpostosAf(
        &$arrayImpostosItem, $filial, $lancamento, $filialDestAf,
        $filialLanc, $nrAutoForn, $dtProgentAf, $operador, $organizacao
    ) {
        $impostosTemp = array();
        foreach ($arrayImpostosItem as $imposto) {
            $cdimposto = $imposto->getCdimpostoit();
            $valorImposto = $imposto->getVrimposit();
            $valorAliq = $imposto->getVraliqimpoit();

            if (array_key_exists($cdimposto, $impostosTemp)) {
                $impostosTemp[$cdimposto]['VRIMPOSTO'] += $valorImposto;
                $impostosTemp[$cdimposto]['VRALIQ'] += $valorAliq;
            } else {
                $impostosTemp[$cdimposto]['VRIMPOSTO'] = $valorImposto;
                $impostosTemp[$cdimposto]['VRALIQ'] = $valorAliq;
            }
        }

        foreach ($impostosTemp as $key => $value) {
            $cdimposto = $key;
            $vrimpitmaf = $value['VRIMPOSTO'];
            $vraliq = $value['VRALIQ'];
            $this->lancamentoEntradaFactory->novoImpostoTemp(
                $cdimposto, $vrimpitmaf, $vraliq, $filial, $lancamento, $operador, $organizacao
            );
        }
    }

    /**
     * Salva os impostos dos itens da af e na variavel '&$arrayImpostos' é armazenado esses impostos
     * para realização do salvar dos impostos da nota que foi gerada(na função 'salvaImpostosAf').
     */
    private function salvaImpostosItemAf($cabecalho, $item,
        /* $filial, $lancamento, $filialDestAf, $filialLanc, $parentInfo,
        $cdproduto, $nrseqitprod, $organizacao, $operador,*/ &$arrayImpostos) {

        $this->calculaImpostosItem($cabecalho, $item);

        $impstemp = Trabitproimpos::findByCdfilimpoitAndNrlancimpoitAndNrseqimpoitAndNrorg($item['CDFILIAL'], $item['NRLANCTONF'], $item['NRSEQITPROD'], $item['NRORG']);
        $arrayImpostos = array_merge($arrayImpostos, $impstemp);
    }

    private function buscaItensAf(&$itensAf, $filialDestAf, $nrAutoForn, $indice = false) {
        $nextItensAf = Itemautf::findByCdfildestAndNrautoforn($filialDestAf, $nrAutoForn);

        foreach($nextItensAf as $nextItem) {
            //Tribute to Delphi's implementation
            $boAchou = false;
            foreach($itensAf as $key => $item) {
                if($item['CDMARCA'] == $nextItem->getCdmarca() && $item['CDPRODUTO'] == $nextItem->getCdproduto() && $item['VRPREUNIAF'] == $nextItem->getVrpreuniaf()) {
                    $boAchou = true;

                    $itensAf[$key]['QTPROGENTAF'] += floatval($nextItem->getQtprogentaf());
                    $itensAf[$key]['QTEFETENTRAF'] += floatval($nextItem->getQtefetentraf());
                    $itensAf[$key]['VRDESCITMAF'] += floatval($nextItem->getVrdescitmaf());
                    $itensAf[$key]['VRPERDESAF'] += floatval($nextItem->getVrperdesaf());

                    $parentInfo = array();
                    $parentInfo['DTPROGENTAF'] = $nextItem->getDtprogentaf();
                    $parentInfo['NRAUTOFORN'] = $nrAutoForn;
                    $parentInfo['QTPROGENTAF'] = floatval($nextItem->getQtprogentaf());

                    $itensAf[$key]['PARENT_INFO'][] = $parentInfo;
                }
            }

            if(!$boAchou) {
                $item = array();
                $item['CDPRODUTO']      = $nextItem->getCdproduto();
                $item['CDMARCA']        = $nextItem->getCdmarca();
                $item['IDBAIXITMAF']    = $nextItem->getIdbaixitmaf();
                $item['QTPROGENTAF']    = floatval($nextItem->getQtprogentaf());
                $item['QTEFETENTRAF']   = floatval($nextItem->getQtefetentraf());
                $item['VRPREUNIAF']     = floatval($nextItem->getVrpreuniaf());
                $item['VRDESCITMAF']    = floatval($nextItem->getVrdescitmaf());
                $item['VRPERDESAF']     = floatval($nextItem->getVrperdesaf());
                
                if($indice) {
                    $item['VRPREUNIAF']= $this->lancamentoEntradaFactory->getVrindiceItmAf($filialDestAf, $nrAutoForn, $nextItem->getCdproduto(), $nextItem->getCdmarca());
                }
                else {
                    $item['VRPREUNIAF'] = floatval($nextItem->getVrpreuniaf());
                }

                $parentInfo = array();
                $parentInfo['DTPROGENTAF'] = $nextItem->getDtprogentaf();
                $parentInfo['NRAUTOFORN'] = $nrAutoForn;
                $parentInfo['QTPROGENTAF'] = floatval($nextItem->getQtprogentaf());

                $item['PARENT_INFO'] = array();
                $item['PARENT_INFO'][] = $parentInfo;

                $itensAf[] = $item;
            }
        }
    }

    /**
     * Busca e soma os encargos das af's
     */
    private function buscaEncargosAf(
        &$arrayEncargos, $filialDestAf, $filialLanc, $nrAutoForn, $dtProgentAf, $organizacao
    ) {
        $dtProgentAfFormat = self::formataData($dtProgentAf);
        $encargosAf = Encargoaf::findByCdfildestAndCdfillancAndNrautofornAndDtentrencafAndNrorg(
            $filialDestAf, $filialLanc, $nrAutoForn, $dtProgentAfFormat, $organizacao
        );

        foreach ($encargosAf as $encargo) {
            $cdencargoaf = $encargo->getCdencargo();
            $vrencargoaf = $encargo->getVrencargoaf();
            if (!array_key_exists($cdencargoaf, $arrayEncargos)) {
                $arrayEncargos[$cdencargoaf] = array(
                    'CDENCARGO' => $cdencargoaf,
                    'VRENCARGO' => $vrencargoaf
                );
            } else
                $arrayEncargos[$cdencargoaf]['VRENCARGO'] += $vrencargoaf;
        }
    }

    /**
     * Salva encargos da AF
     */
    private function salvaEncargosAf($arrayEncargosAf, $filial, $lancamento, $organizacao, $operador) {
        try {
            $arrayEncargos = array();
            $encargosPadrao = Encargo::findByNrorg($organizacao);
            foreach ($encargosPadrao as $key => $encargo) {
                $cdencargo = $encargo->getCdencargo();
                $valor = array_key_exists($cdencargo, $arrayEncargosAf) ? $valor = $arrayEncargosAf[$cdencargo]['VRENCARGO'] : 0;

                $arrayEncargos[] = $this->lancamentoFactory->novoEncargo(
                    $filial, $lancamento, $cdencargo, $valor, $organizacao, $operador
                );
                $this->entityManager->flush();
            }
            return $arrayEncargos;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
    *   Atualiza o item da af quando a nota é editada
    */
    public function atualizaItemAf($item) {
        $itemNota = Itemproduto::findByCdfilialAndNrlanctonfAndNrseqitprodAndNrorg($item["CDFILIAL"], $item["NRLANCTONF"], $item["NRSEQITPROD"], $this->environment->getNrOrgTrab());
        $itemNota = $itemNota[0];

        if($itemNota->getNrautoforn()) {
            $this->lancamentoEntradaFactory->atualizaStatusAF($itemNota, self::ALTERACAO, $item["OLDQTITEMPROD"]);
        }
        
    }
    
    /**
     *  Exclui item af
     */
    public function excluirItemAf($item) {
        if($item["NRAUTOFORN"]) {
            $this->lancamentoEntradaFactory->excluirItemAf($item);
        }
    } 

    /***************************** / Nota Fiscal - Importação AF ***************************************/

    /*********************************** Nota Fiscal - Imposto ******************************************/
    public function salvaImpostosNotaFiscal(array $impostos) {
        try {
   
            foreach ($impostos as $imposto) {
                $filial = $imposto['CDFILIMPTOT'];
                $nrlanctonf = $imposto['NRLANCIMPTOT'];
                $operador = $this->environment->getCdOperador();
                $organizacao = $this->environment->getNrOrgTrab();
                $cdimposto = $imposto['CDIMPOSTOT'];

                $trabImposnfTot = Trabimposnftot::findOneByCdfilimptotAndNrlancimptotAndCdimpostot($filial, $nrlanctonf, $cdimposto);

                if (is_object($trabImposnfTot)) {
                    $vrimposto = $imposto['VRIMPOSTOT'];
                    $vrbcalimptot = $trabImposnfTot->getVrbcalimptot();
                    $iddifaliqcusptn = $trabImposnfTot->getIddifaliqcusptn();
                    $iderrostot = $trabImposnfTot->getIderrostot();
                    $idavisostot = $trabImposnfTot->getIdavisostot();
                    //$idativo = $trabImposnfTot->getIdativo();

                    $this->lancamentoEntradaFactory->editarImpostoTemp(
                        $trabImposnfTot, $vrimposto, $vrbcalimptot, $iddifaliqcusptn,
                        $iderrostot, $idavisostot, $operador, $organizacao
                    );
                } else {
                    $filial     = $imposto['CDFILIMPTOT'];
                    $lancamento = $imposto['NRLANCIMPTOT'];
                    $vrimposto  = $imposto['VRIMPOSTOT'];
                    $vraliq     = 0;

                    $this->lancamentoEntradaFactory->novoImpostoTemp(
                        $cdimposto, $vrimposto, $vraliq, $filial, $lancamento, $operador, $organizacao
                    );
                }
            }
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function removeTrabsLancamentoEntrada($filial, $lancamento) {
        try {
            $organizacao = $this->environment->getNrOrgTrab();

            $imposnftot = Imposnftot::findByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);
            $itemproimpos = Itemproimpos::findByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);

            if (is_array($imposnftot)) {
                foreach ($imposnftot as $imposto) {
                    $imposto->delete();
                }
            }

            if (is_array($itemproimpos)) {
                foreach ($itemproimpos as $imposto) {
                    $imposto->delete();
                }
            }

            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaNotaCadastrada($filial, $serie, $nrinsjurnf, $nrnotafisc) {
        try {
            $countNotasCadastradas = $this->lancamentoEntradaFactory->verificaNotaCadastrada(
                $filial, $nrnotafisc, $serie, $nrinsjurnf
            );

            if ($countNotasCadastradas > 0)
                throw new \Exception('Nota fiscal já cadastrada para remetente.');

        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function validaEditNotaCadastrada($filial, $serie, $nrinsjurnf, $nrnotafisc, $nrlanctonf){
        try {
            $nfs = $this->lancamentoEntradaFactory->verificaEditNotaCadastrada(
                $filial, $serie, $nrinsjurnf, $nrnotafisc
            );
            
            $arrayNfRepetidas = [];
            
            forEach($nfs as $nf){
                if($nf["NRLANCTONF"] != $nrlanctonf){
                    array_push($arrayNfRepetidas, $nf["NRLANCTONF"]);
                }
            }
            
            if(count($arrayNfRepetidas) > 0)
                    throw new \Exception('Nota fiscal já cadastrada para remetente.');
                    
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /************************** Nota Fiscal - Importação Chave Acesso ************************************/
    public function realizaImportacaoChaveAcesso($filial, $lancamento, $nracessonfe, $rowLancamento) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            //TODO nao instanciar a classe
            $chaveAcesso = new ChaveAcesso($this->entityManager, $filial, $lancamento, $nracessonfe, $this->lancamentoService, $rowLancamento, $this->environment);
            $chaveAcesso->realizaImportacao();
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    /************************** Nota Fiscal - Importação AF e Chave Acesso ************************************/
    public function realizaImportacaoAfChaveAcesso($filial, $lancamento, $nracessonfe, $afSelecionadas, $rowLancamento) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $operador = $this->environment->getCdOperador();
            $organizacao = $this->environment->getNrOrgTrab();
            $produtosNaoCadastradosAf = array();

            //TODO nao instanciar a classe
            $chaveAcesso = new ChaveAcesso($this->entityManager, $filial, $lancamento, $nracessonfe, $this->lancamentoService, $rowLancamento, $this->environment);
            $chaveAcesso->realizaImportacao();
            $this->entityManager->flush();
            $produtosCadastrados = array();

            $itensNotaFiscal = Itemproduto::findByCdfilialAndNrlanctonf($filial, $lancamento);
            
            $notaFiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);
    
            foreach ($itensNotaFiscal as $item) {
                $cdproduto = $item->getCdproduto();
                $itemAf = null;
                $produtoNaoCadastrados = !isset($produtosNaoCadastradosAf[$cdproduto]);
                
                foreach ($afSelecionadas as $af) {
                        $filialLanc = $af['CDFILLANC'];
                        $filialDestAf = $af['CDFILDEST'];
                        $nrAutoForn = $af['NRAUTOFORN'];
                        $dtProgentAf = $af['DTPROGENTAF'];
                        $dtAutoForn = $af['DTAUTOFORN'];
                        $vrTotalAf = $af['VRTOTALAF'];
                        
                        $itemAf = Itemautf::findOneByCdfildestAndNrautofornAndCdproduto($filialDestAf, $nrAutoForn, $cdproduto);
                        if (!!$itemAf) {
                            //TODO REFATORAR
                            $cdmarca = $itemAf->getCdmarca();
                            $item->setCdmarca($cdmarca);
                            $item->setNrautoforn($itemAf->getNrautoforn());

                            $dtprogentaf = $itemAf->getDtprogentaf()->format('d/m/Y');

                            $item->setDtprogentaf($itemAf->getDtprogentaf());
                            
                            $qtprogentaf = floatval($itemAf->getQtprogentaf());
                            $vrpreuniaf = $itemAf->getVrpreuniaf();
                            $qtnfefetaf = $item->getQtitemprod();
                            if ($produtoNaoCadastrados) {
                                //Cria o relacionamento entre os produtos da AF e a nota fiscal.
                                $this->novoRelacionamentoAfNf(
                                    $filial, $lancamento, $filialDestAf, $filialLanc, $nrAutoForn,
                                    $cdmarca, $cdproduto, $dtProgentAf, $qtprogentaf, $vrpreuniaf,
                                    $qtnfefetaf, $organizacao, $operador
                                );
                            }
                            else {
                                $this->lancamentoEntradaFactory->atualizaRelacionamentoAfNf($filial, $lancamento, $filialDestAf, $filialLanc, $nrAutoForn,
                                    $cdmarca, $cdproduto, $qtnfefetaf);
                            }

                        }
                    }
                
                $produtosNaoCadastradosAf[$cdproduto] = $cdproduto;
                
            }

            // Chama procedure NF_BAIXA_AF para dar baixa do item na AF
            $this->entityManager->flush();
            
            $this->atualizaStatusAF($filial, $lancamento, $organizacao, 'I');
            
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public function salvaRelacionamentoItensXml(
        $cdprodutointg, $cdforneced, $cdprodfornxml, $nmprodutointg, $cdfilial,$CDPRODUTOXML_BACKUP_ROW
    ) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $operador = $this->environment->getCdOperador();
            $organizacao = $this->environment->getNrOrgTrab();

           
            
            try {
                $utilizaRelacionamentoFilial = $this->lancamentoEntradaFactory->utilizaRelacionamentoPorFilial();
                if(isset($utilizaRelacionamentoFilial["IDUTRELXMLFIL"])) {
                    $utilizaRelacionamentoFilial = $utilizaRelacionamentoFilial["IDUTRELXMLFIL"] == 'S';
                }
                else {
                    $utilizaRelacionamentoFilial = false;
                }
            }
            catch(\Exception $e) {
                $utilizaRelacionamentoFilial = false;
            }
            
            
            if($utilizaRelacionamentoFilial) {
                $prodfornxml = $this->lancamentoEntradaFactory->buscaDeleteProdfornxmlFilial($cdforneced, $cdprodutointg, $cdfilial, $CDPRODUTOXML_BACKUP_ROW);
                $this->entityManager->flush();
                
                $this->lancamentoEntradaFactory->novoProdfornxmlFilial(
                    $cdprodfornxml, $cdforneced, $cdprodutointg, $nmprodutointg, $cdfilial, $organizacao, $operador
                );
            }
            else {
                $prodfornxml = $this->lancamentoEntradaFactory->buscaDeleteProdfornxml($cdforneced, $cdprodutointg);
                $this->entityManager->flush();
                
                $this->lancamentoEntradaFactory->novoProdfornxml(
                    $cdprodfornxml, $cdforneced, $cdprodutointg, $nmprodutointg, $organizacao, $operador
                );
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }
    
    public function deleteProdutos($cdfilial, $nrlanctonf,$rowRelacionamento){
        foreach($rowRelacionamento as $row){
           $nomeProdFornxml = $row['NMPRODUTOFORNXML'];
           $cdProdEmis = $row['CDARVPRODFORNXML'];
           $this->lancamentoEntradaFactory->deleteProdutos($cdfilial, $nrlanctonf,$cdProdEmis, null);
        }
    }

    public function atualizaDadosProduto($cdfilial, $nrlanctonf,$rowRelacionamento){
        
        $maxSeqItem =  $this->lancamentoEntradaFactory->getMaxSeqItem($cdfilial, $nrlanctonf);
            
        if(!$maxSeqItem) {
            $seqItem = 0;
        }
        else {
            $seqItem = $maxSeqItem + 1;
        }
        
        foreach($rowRelacionamento as $item) {
            $nrseqitem = str_pad($seqItem, 4, '0', STR_PAD_LEFT);
    
            $seqItem = $nrseqitem;
            $produto = $item['CDPRODFORNXML'];
            $codEmissao = $item['CDARVPRODFORNXML'];
            $descEmissao = $item['NMPRODUTOFORNXML'];
            $quantidade = $item['QTITEMPROD'];
            $valorUnitario = $item['VRUNITITEM'];
            $valorDesconto = $item['VRDESCITEM'];
            $almoxarifado = array_key_exists("CDALMOXARIFE", $item) ? $item["CDALMOXARIFE"] : null;
            $localEstoque = array_key_exists("CDLOCALESTOQ", $item) ? $item["CDLOCALESTOQ"] : null;
            $nrloteestq = array_key_exists("NRLOTEESTQ", $item) ? $item['NRLOTEESTQ'] : null;
            $nrsublote = array_key_exists("NRSUBLOTE", $item) ? $item['NRSUBLOTE'] : null;
            
            $this->lancamentoService->procedureGeraNotaFiscalProduto(
                
                $cdfilial, $nrlanctonf, $produto, $codEmissao, $descEmissao, $quantidade, $valorUnitario, $valorDesconto,
                $almoxarifado, $localEstoque, $nrloteestq, $nrsublote
            );
            
            $seqItem += 1;
               
        }
       
    }

    public function removerRelacionamentoItemXml($cdforneced, $cdprodutoxml, $filial, $cdproduto) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            try {
                $utilizaRelacionamentoFilial = $this->lancamentoEntradaFactory->utilizaRelacionamentoPorFilial();
                if(isset($utilizaRelacionamentoFilial["IDUTRELXMLFIL"])) {
                    $utilizaRelacionamentoFilial = $utilizaRelacionamentoFilial["IDUTRELXMLFIL"] == 'S';
                }
                else {
                    $utilizaRelacionamentoFilial = false;
                }
            }
            catch(\Exception $e) {
                $utilizaRelacionamentoFilial = false;
            }
            
            if($utilizaRelacionamentoFilial) {
                 $prodfornxml = $this->lancamentoEntradaFactory-> buscaDeleteProdfornxmlFilial($cdforneced, $cdprodutoxml, $filial, $cdproduto);
               
            }else{
                  $prodfornxml = $this->lancamentoEntradaFactory->buscaDeleteProdfornxml($cdforneced, $cdprodutoxml);
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public function calculaImpostosItem($cabecalho, $item) {
        $filial = $item['CDFILIAL'];
        $nrlanctonf = $item['NRLANCTONF'];
        $seqItem = $item['NRSEQITPROD'];
        $impostosItem = $this->lancamentoService->calculaImpostoItemProduto($cabecalho, $item);
        foreach ($impostosItem as &$imposto) {
            $imposto['CDFILIAL'] = $filial;
            $imposto['NRLANCTONF'] = $nrlanctonf;
            $imposto['NRSEQITPROD'] = $seqItem;
        }
        return $impostosItem;
    }

    public function novoMultiItemProduto($itens, $cabecalho, &$impostosTemp) {
        foreach ($itens as $item) {
            $filial = $item['CDFILIAL'];
            $nrlanctonf = $item['NRLANCTONF'];
            $seqItem = $item['NRSEQITPROD'];
            $produto = $item['CDPRODUTO'];
            $codEmissao = $item['CDPRODEMIS'];
            $descEmissao = $item['DSPRODEMIS'];
            $quantidade = $item['QTITEMPROD'];
            $valorUnitario = $item['VRUNITITEM'];
            $valorDesconto = $item['VRDESCITEM'];
            $almoxarifado = array_key_exists("CDALMOXARIFE", $item) ? $item["CDALMOXARIFE"] : null;
            $localEstoque = array_key_exists("CDLOCALESTOQ", $item) ? $item["CDLOCALESTOQ"] : null;
            $nrloteestq = array_key_exists("NRLOTEESTQ", $item) ? $item['NRLOTEESTQ'] : null;
            $nrsublote = array_key_exists("NRSUBLOTE", $item) ? $item['NRSUBLOTE'] : null;
            
            $this->lancamentoService->procedureGeraNotaFiscalProduto(
                
                $filial, $nrlanctonf, $produto, $codEmissao, $descEmissao, $quantidade, $valorUnitario, $valorDesconto,
                $almoxarifado, $localEstoque, $nrloteestq, $nrsublote
            );

            $item['__generate_taxes'] = true;

            $impostosTemp[] = $this->calculaImpostosItem($cabecalho, $item);
        }
    }

    public function editMultiItemProduto($itens, $cabecalho, &$impostosTemp) {
        foreach ($itens as $item) {
            $filial = $item['CDFILIAL'];
            $nrlanctonf = $item['NRLANCTONF'];
            $produto = $item['CDPRODUTO'];
            $seqItem = $item['NRSEQITPROD'];
            $codEmissao = $item['CDPRODEMIS'];
            $descEmissao = $item['DSPRODEMIS'];
            $quantidade = $item['QTITEMPROD'];
            $valorUnitario = $item['VRUNITITEM'];
            $valorDesconto = $item['VRDESCITEM'];
            $valorTotal = $item['VRTOTAITEM'];
            $organizacao = $this->environment->getNrOrgTrab();
            $operador = $this->environment->getCdOperador();

            $itemproduto = Itemproduto::findOneByCdfilialAndNrlanctonfAndCdprodutoAndNrseqitprod(
                $filial, $nrlanctonf, $produto, $seqItem
            );

            $vrbrutitem = $itemproduto->getVrbrutitem();
            $nrautoforn = $itemproduto->getNrautoforn();
            $cdfilimovi = $itemproduto->getCdfilimovi();
            $cdmarca = $itemproduto->getCdmarca();

            $cdalmoxarife = $itemproduto->getCdalmoxarife();
            $cdlocalestoq = $itemproduto->getCdlocalestoq();

            $nrloteestq = $itemproduto->getNrloteestq();
            $nrsublote = $itemproduto->getNrsublote();

            $idatuvrestoq = $itemproduto->getIdatuvrestoq();
            $vrcontitem = $itemproduto->getVrcontitem();
            $idconforitem = $itemproduto->getIdconforitem();

            $this->lancamentoFactory->editarItemproduto(
                $itemproduto, $codEmissao, $descEmissao, $quantidade, $vrbrutitem, $valorUnitario,
                $valorDesconto, $valorTotal, $organizacao, $operador
            );

            $impostosTemp[] = $this->calculaImpostosItem($cabecalho, $item);
        }
    }
    

    public function salvaMultiItensLancamentoEntrada($novosItens, $itensAlterados, $cabecalho) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $impostosTemp = array();

            $this->novoMultiItemProduto($novosItens, $cabecalho, $impostosTemp);
            $this->editMultiItemProduto($itensAlterados, $cabecalho, $impostosTemp);

            $this->entityManager->flush();
            $connection->commit();

            return $impostosTemp;
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    private function verificaUsoNotaOrigem($lancamento) {
        $usaNotaOrigem = ($lancamento['IDOPERDEVONF'] == 'S');
        $temNotaOrigem = (isset($lancamento['DSNOTAFISCORIG']));
        if($usaNotaOrigem) {
            if(!$temNotaOrigem)
                throw new \Exception(($usaNotaOrigem)? "NF_ORIGEM_NECESSARIA" : "NF_ORIGEM_NAO_NECESSARIA");
        }

        return $usaNotaOrigem;
    }

    public function getNFOrigemDevolucao($CDFILIORIG, $IDTPIJURNF, $NRINSJURNF, $CDFILIRECE, $CDSERIE,
                              $NRNOTAFISC, $CDTIPOOPEN, $CDTIPOOPSA, $CDTIPOOPER, $RELACDOORI, $IDNFESTCANC) {
        if($IDNFESTCANC == 'N' || $IDNFESTCANC == '') {
            $params = [
                "CDFILIORIG" => $CDFILIORIG,
                "IDTPIJURNF" => $IDTPIJURNF,
                "NRINSJURNF" => $NRINSJURNF,
                "CDFILIRECE" => $CDFILIRECE,
                "CDSERIE" => $CDSERIE,
                "NRNOTAFISC" => $NRNOTAFISC,
                "CDTIPOOPEN" => $CDTIPOOPEN,
                "CDTIPOOPSA" => $CDTIPOOPSA,
                "CDTIPOOPER" => $CDTIPOOPER,
                "RELACDOORI" => $RELACDOORI
            ];

            return $this->lancamentoEntradaFactory->getNFOrigemDevolucao($params);
        } else { //encargo
            throw new \Exception("Not implemented yet.");
        }
    }

    public function validaNotaDeOrigem($lancamento) {
        if(isset($lancamento['IDDCORIGRELAC'])) 
            $IDDCORIGRELAC = $lancamento['IDDCORIGRELAC'];
        else 
            $IDDCORIGRELAC = '';
            
        if(isset($lancamento['IDNFESTCANC'])) 
            $IDNFESTCANC = $lancamento['IDNFESTCANC'];
        else 
            $IDNFESTCANC = '';
            

        if($this->verificaUsoNotaOrigem($lancamento)) {
            if($lancamento['IDORIGDESTNF'] == 'C' || $lancamento['IDORIGDESTNF'] == 'F') {
                if(isset($lancamento['IDTPIJURNF']))
                    $IDTPIJURNF = $lancamento['IDTPIJURNF'];
                else 
                    $IDTPIJURNF = '';
                $NRINSJURNF = $lancamento['NRINSJURNF'];
                $CDFILIAL = '';
                $CDFILIORIG = $lancamento['CDFILIRECE'];
            } else {
                $IDTPIJURNF = '';
                $NRINSJURNF = '';

                if($lancamento['IDNFESTCANC'] == 'S') {
                    $CDFILIAL = $lancamento['CDFILIEMIT'];
                    $CDFILIORIG = $lancamento['CDFILIRECE'];
                } else {
                    $CDFILIAL = $lancamento['CDFILIRECE'];
                    $CDFILIORIG = $lancamento['CDFILIEMIT'];
                }
            }

             if($lancamento['IDOPERDEVONF'] == 'S' && $lancamento['CDTIPOOPER'] != 3) {
                $erros = 0;

                if(!is_array($lancamento['DSNOTAFISCORIG']))
                    $lancamento['DSNOTAFISCORIG'] = array($lancamento['DSNOTAFISCORIG']);
                    
                
                foreach($lancamento['DSNOTAFISCORIG'] as $serie_numero) {
                    $serie_numero = explode('-', $serie_numero);
                    $serieNFOrig = $serie_numero[0];
                    $nrNFOrig = $serie_numero[1];
                    
                    
                    $NFOrigem = $this->getNFOrigemDevolucao($CDFILIORIG, $IDTPIJURNF, $NRINSJURNF, $CDFILIAL,
                                                            $serieNFOrig, $nrNFOrig, $lancamento['CDTIPOOPEN'],
                                                            $lancamento['CDTIPOOPSA'], $lancamento['CDTIPOOPER'],
                                                            $IDDCORIGRELAC, $IDNFESTCANC);

                    if(is_null($NFOrigem))
                        $erros++;
                }
                
                if($erros > 0)
                    throw new \Exception(($erros == 1)? "NF_ORIGEM_NAO_CADASTRADA" : "NFS_ORIGEM_NAO_CADASTRADAS");
            }
        }
    }
    
    public function verificaNotaXml($cdFilial, $nrLanctonf) {
        $notaXml = $this->lancamentoEntradaFactory->verificaNotaXml($cdFilial, $nrLanctonf);
        if(isset($notaXml[0])) {
            if($notaXml[0]["IDSTATUSNFE"] == 'U') return true;
            else return false;
        }
        else return false;
    }
    
    public function aprovaNfXmlAutomatica() {
        $paramGeral = $this->lancamentoEntradaFactory->aprovaNfXmlAutomatica();
        if(isset($paramGeral[0])) {
            if($paramGeral[0]["IDPERAPROVNFXML"] == 'S') return true;
            else return false;
        }
        else return false;
    }
    
    public function limpaCapa($cdfilial, $nrlanctonf) {
        return $this->lancamentoEntradaFactory->limpaCapa($cdfilial, $nrlanctonf);
    }
    
    public function validaInscEstadual($request, $response){

        $rowLancamento = $request;
        $rowLancamento["NRACESSONFE"] = str_replace(".", "", $rowLancamento["NRACESSONFE"]);
        
        if($rowLancamento["IDOPFATELE"] != 'S') {
            $cdfilial = $rowLancamento["CDFILIRECE"];
        
            $organizacao = $this->environment->getNrOrgTrab();
            $xml = IntgNotafiscal::findOneByNracessonfe($rowLancamento["NRACESSONFE"]);
            $inscrEstad = Inscrestad::findByCdinscestaAndCdempresaAndNrorg($rowLancamento["CDINSCESTA"], $rowLancamento["CDEMPRESA"], $organizacao);
    
            if(is_array($inscrEstad)) {
                $inscrEstad = $inscrEstad[0];
            }
            
            $xmlCdInscesta   = $xml->getCdinscesta();
            $xmlCdFilirece   = $xml->getCdfilirece();
            $xmlNrInsjurempr = $this->lancamentoEntradaFactory->getNrinsjurest($rowLancamento["NRACESSONFE"]);
            $nfCdInscesta    = $rowLancamento["CDINSCESTA"];
            $nfCdFilirece    = $rowLancamento["CDFILIRECE"];
            //Checa se filial foi selecionada
            if(!$nfCdFilirece)
                return false;
                
            if($this->lancamentoEntradaFactory->getIdbloqxmlinsdif() == 'N'){
                
                $cdtipooper = $rowLancamento["CDTIPOOPER"];
                $idvalidinsxml = $this->lancamentoEntradaFactory->getIdValidInsXML($cdtipooper, $cdfilial);
                
                if(is_null($idvalidinsxml) || $idvalidinsxml == 'S'){
                    if($xmlCdFilirece){
                       if($xmlCdFilirece != $nfCdFilirece){
                            throw new \Exception("Operação bloqueada. Nota fiscal não pertence a filial informada.");
                        }
                    }
                    if($xmlCdInscesta){
                        if($xmlCdInscesta == 'ISENTO'){
                            if(strtoupper(substr($xmlCdInscesta, 0, 6)) != $nfCdInscesta){
                                throw new \Exception("Operação bloqueada. Nota fiscal não pertence a filial informada.");
                            } else{
                                if(str_pad($xmlCdInscesta, 20, '0', STR_PAD_LEFT) != str_pad($nfCdInscesta, 20, '0', STR_PAD_LEFT)){
                                    throw new \Exception("Operação bloqueada. Nota fiscal não pertence a filial informada.");
                                }
                            }
                        }
                    }else{
                        if($xmlNrInsjurempr){
                            if($xmlNrInsjurempr != $inscrEstad->getNrinsjuriest()){
                                throw new \Exception("Operação bloqueada. Nota fiscal não pertence a empresa informada.");
                            }
                        }
                    }
                }
                
            } else{
                if($xmlCdFilirece){
                    if(str_pad($xmlCdFilirece, 4, '0', STR_PAD_LEFT) != str_pad($nfCdFilirece, 4, '0', STR_PAD_LEFT)){
                        throw new \Exception("Operação bloqueada. Nota fiscal não pertence a filial informada.");
                    }
                }
                if($xmlCdInscesta) {
                    if($xmlCdInscesta == 'ISENTO') {
                        if(strtoupper(substr($xmlCdInscesta, 0, 6)) != $nfCdInscesta){
                            throw new \Exception("Operação bloqueada. Nota fiscal não pertence a filial informada.");
                        } 
                    } else {
                        if(str_pad($xmlCdInscesta, 20, '0', STR_PAD_LEFT) != str_pad($nfCdInscesta, 20, '0', STR_PAD_LEFT)){
                            throw new \Exception("Operação bloqueada. Nota fiscal não pertence a filial informada.");
                        }
                    }
                } else{
                    if($xmlNrInsjurempr){
                        if($xmlNrInsjurempr != $inscrEstad->getNrinsjuriest()){
                            throw new \Exception("Operação bloqueada. Nota fiscal não pertence a empresa informada.");
                        }
                    }
                }
            }
        }
        
        $response->addDataset(new DataSource\DataSet("valido", array(true)));
    }
    
    
    public function getDefaultFilter($cdFilial) {
        $cdFilial = $this->environment->getCdfilial() ? $this->environment->getCdfilial() : $cdFilial;
        $filter = $this->lancamentoEntradaFactory->getDefaultFilter($cdFilial);
        if(!empty($filter)) {
            if($filter["DTPROCESSA"]) {
                $filter["DTENTRSAID"] = array("start" => $filter["DTPROCESSA"], "end" => $filter["DTPROCESSA"]);
            }
            else {
                $filter["DTENTRSAID"] = array("start" => date("d/m/Y"), "end" => date("d/m/Y"));
            }
        }
        return $filter;
    }
    
    public function getDefaultNovaNota($cdFilial) {
        $cdFilial = $cdFilial ? $cdFilial : $this->environment->getCdfilial();
        $cdOperador = $this->environment->getCdoperador();
        $default = $this->lancamentoEntradaFactory->getDefaultNovaNota($cdFilial, $cdOperador);
        if(!empty($default)) {
            if($default["DTPROCESSA"]) {
                $default["DTENTRSAID"] = substr($default["DTPROCESSA"], 0, 10);
                $default["DTEMISSAO"]  = substr($default["DTPROCESSA"], 0, 10);
            }
            else {
                $default["DTENTRSAID"] = date("d/m/Y");
                $default["DTEMISSAO"]  = date("d/m/Y");
            }
            $default["CDFILIAL"] = $this->environment->getCdfilial();
        }
        else {
            $default = array("CDFILIAL" => $this->environment->getCdfilial());
        }
        $default["OPERADOR"] = $this->environment->getCdOperador();
        $default["OPERGERA"] = $this->environment->getCdOperador();
        $default["CDFILIAL"] = $this->environment->getCdFilial();
        return $default;
    }
    
    public function getIDATUESTTPOP($row) {
        //$cdFilial = $cdFilial ? $cdFilial : $this->environment->getCdfilial();
        //var_dump($row);die;
        $identrsaidop = $row['IDENTRSAIDOP'];
        $cdtipooper = $row['CDTIPOOPER'];
        $cdOperador = $this->environment->getCdoperador();
        
        $atualizaEstoque = $this->lancamentoEntradaFactory->getIDATUESTTPOP($identrsaidop, $cdtipooper);
        
        //var_dump($atualizaEstoque);die;

        return $atualizaEstoque;
    }
    
    public function validaCamposChaveAcesso($lancamento) {
        $nracessonfe = $lancamento["NRACESSONFE"];
        
        $cnpjChave      = substr($nracessonfe, 6, 14);
        $numeroChave    = substr($nracessonfe, 25, 9);
        $serieChave     = substr($nracessonfe, 22, 3);
        $mesChave       = substr($nracessonfe, 4, 2);
        $anoChave       = substr($nracessonfe, 2, 2);
        
        if($lancamento["NRINSJURNF"] != $cnpjChave) {
            if($this->lancamentoEntradaFactory->fornecedorVerificaChave($lancamento["CDFORNECED"]) == 'S') {
                throw new \Exception("Chave de acesso NF-e inválida ou não pertence a nota fiscal informada.");
            }
            if($lancamento["NRNOTAFISC"]   != $numeroChave
                 || $lancamento["CDSERIEENTR"]      != (int) $serieChave 
                 || substr($lancamento["DTEMISSAO"], 3, 2) != $mesChave
                 || substr($lancamento["DTEMISSAO"], 8, 2) != $anoChave) {
                 throw new \Exception("Chave de acesso NF-e inválida ou não pertence a nota fiscal informada.");
            }
        }
        else if($lancamento["NRNOTAFISC"]   != $numeroChave
             || $lancamento["CDSERIEENTR"]      != (int) $serieChave 
             || substr($lancamento["DTEMISSAO"], 3, 2) != $mesChave
             || substr($lancamento["DTEMISSAO"], 8, 2) != $anoChave) {
             throw new \Exception("Chave de acesso NF-e inválida ou não pertence a nota fiscal informada.");
        }
    }
    
    
    public function validacoesItens($rowLancamento) {
        $this->validaRelacionamentoItens($rowLancamento["NRACESSONFE"], $rowLancamento["CDFORNECED"], $rowLancamento["CDFILIRECE"]);
        if(isset($rowLancamento["AF"])) $this->validaAf($rowLancamento["CDFILIRECE"], $rowLancamento["AF"]);
    }
    
    protected function validaRelacionamentoItens($chaveAcesso, $fornecedor, $filial) {
        $params = $this->getParamsRelacItens($filial, $fornecedor);
        if($params["IDFUNCFILI"] != 'O' || $params["IDUTPRIGXML"] === 'N'){
            if($this->lancamentoEntradaFactory->existeItemSemRelacionamento($chaveAcesso, $fornecedor, $filial)) 
               throw new \Exception("Existe(m) produto(s) que não possue(m) relacionamento.");
        }
    }
    
    public function validaModeloDocumento($idEntrsaidop, $cdttipooper, $cdmoddocfis) {
        return $this->lancamentoEntradaFactory->validaModeloDocumento($idEntrsaidop, $cdttipooper, $cdmoddocfis);
    }
    
    protected function validaAf($cdFilial, $afs) { 
        if(empty($afs)) {
            if($this->lancamentoEntradaFactory->verificaAfObrigatoria()) 
                throw new \Exception("Produtos somente poderão ser lançados na Nota Fiscal quando os mesmos forem oriundos de uma Autorização de Fornecimento.");
        }
/*        else {
            $nrAutoForn = $this->montaIn($afs);
            $itensAf = $this->lancamentoEntradaFactory->getItensAf($cdFilial, $nrAutoForn);
            $itens   = array();
            foreach($itensAf as $item) {
                array_push($itens, $item["CDPRODUTO"]);
            }

        }*/
    }
    
    private function getParamsRelacItens($cdfilial, $cdforneced){
        $params = array(
            "IDUTPRIGXML"   => $this->lancamentoEntradaFactory->getProdIgXml($cdforneced),
            "IDFUNCFILI"  => $this->lancamentoEntradaFactory->getFuncFili($cdfilial)
        );
        
        return $params;
    }
    
    private function montaIn($afs) {
        $sql = '(';

        foreach($afs as $af) {
            $sql.= "'".$af["NRAUTOFORN"]."'".',';
        }
        

        $size = strlen($sql);
        return substr($sql,0, $size-1).')';
    }
    
    public function getItensNotaOrigem($notaorig, $filiOrig, $nrlanctonf, $cdfilial) {
        return $this->lancamentoEntradaFactory->getItensNotaOrigem($notaorig, $filiOrig, $nrlanctonf, $cdfilial);
    }
    
    public function getFirstItem($filial, $lancamento) {
        return $this->lancamentoEntradaFactory->getFirstItem($filial, $lancamento);
    }
    
    public function calculaImpostoItemProduto($filial, $lancamento) {
        return $this->lancamentoService->calculaImpostoItemProduto($filial, $lancamento);
    }
    
    public function atualizaIntgNotaFiscal($idStatusNfe, $idImportada, $idUpdataData, $cdFiliRece, $cdFilial, $nrLanctonf, $nrAcessoNfe) {
        return $this->lancamentoService->atualizaIntgNotaFiscal($idStatusNfe, $idImportada, $idUpdataData, $cdFiliRece, $cdFilial, $nrLanctonf, $nrAcessoNfe);
    } 
    
    public function getSequencialHistoricoXML($nrAcessoNfe) {
        return $this->lancamentoService->getSequencialHistoricoXML($nrAcessoNfe);     
    }
    
    public function atualizaNotaDestinada($filial, $lancamento, $chave) {
        return $this->lancamentoService->atualizaNotaDestinada($filial, $lancamento, $chave);
    }
    
    public function insereHistoricoXML($nrAcessoNfe, $nrSeqHistor, $dtHistorico, $idTipoOper, $idStatus, $nrProcoloNFe, $dsOperacao, $cdFilial, $nrLanctonf) {
        return $this->lancamentoService->insereHistoricoXML($nrAcessoNfe, $nrSeqHistor, $dtHistorico,
        $idTipoOper, $idStatus, $nrProcoloNFe, $dsOperacao, $cdFilial, $nrLanctonf, $this->environment->getCdoperador(), $this->environment->getNrOrgTrab());   
    }
    
    public function verificaNotaSendoUtilizada($filial, $lancamento) {
        return $this->lancamentoEntradaFactory->verificaNotaSendoUtilizada($filial, $lancamento);
    }
    
    public function getVencimentos($filial, $lancamento) {
        return $this->lancamentoEntradaFactory->getVencimentos($filial, $lancamento);
    }
    
    public function salvaInfoCompra($row) {
        $filial = $row["CDFILIAL"];
        $lancamento = $row["NRLANCTONF"];
        $notaEmp = $row["DSNOTAEMPENHO"];
        $pedido = $row["CDPEDIDO"];
        $contrato = $row["CDCONTRATO"];
        $nrorg = $row["NRORG"];
        if($row['__is_new'] === true){
            $this->lancamentoEntradaFactory->salvaInfoCompra($filial, $lancamento, $notaEmp,$pedido,$contrato,$nrorg);
        }else{
             $this->lancamentoEntradaFactory->atualizaInfoCompra($filial, $lancamento, $notaEmp,$pedido,$contrato,$nrorg);
        }
    }
    
    public function insereItemNotaFisc($filial, $lancamento, $filiOrig, $lancOrig) {
        return $this->lancamentoEntradaFactory->insereItemNotaFisc($filial, $lancamento, $filiOrig, $lancOrig, $this->environment->getNrOrgTrab());
    }
    
    public function converterValorMoeda($vrIndice, $preco){
        return $preco * $vrIndice;
    }
    
    public function validaAFSalvarNota($cdfilial, $cdtipooper, $cdforneced){
        $tipoOperacCompra = '01';
        
        $idutbloqopaf = $this->lancamentoEntradaFactory->getFilialAF($cdfilial);
        $exigeAF = $this->lancamentoEntradaFactory->verificaExigeAf($cdfilial, $cdtipooper, $idutbloqopaf);
        $paramProdutosAF = $this->lancamentoEntradaFactory->verificaParamProdutosAF($cdfilial);
        
        $bloqueiaGeral = $this->lancamentoEntradaFactory->validaBloqueioGeral();
        $bloqueiaFilial = $this->lancamentoEntradaFactory->validaBloqueioFilial($cdfilial);
        $bloqueiaFilialForn = $this->lancamentoEntradaFactory->validaBloqueioFilialForn($cdfilial);
        $bloqueiaFornecedor = $this->lancamentoEntradaFactory->validaBloqueioFornecedor($cdforneced);

        $obrigaAF = false;
        
        // validações feitas somentes para NF Compra
        if($cdtipooper === $tipoOperacCompra){
            if($bloqueiaGeral === 'S'){
                $obrigaAF = true;
            } else if($bloqueiaFilial === 'S'){
              $obrigaAF = true;  
            } else if($bloqueiaFilialForn === 'S') {
                if($bloqueiaFornecedor === 'S') $obrigaAF = true;
            }
        }

        if(!$paramProdutosAF && $exigeAF === 'S'){
            $obrigaAF = true;
        }
        
        return $obrigaAF;
    }
    
    public function atualizaVrItemProduto($nota) {
        
        $dtrefind = substr($nota['DTEMISSAO'], 0, 10);

        if($dtrefind) {

            $dtrefind =  \DateTime::createFromFormat("d/m/Y", $dtrefind);
            $dtrefind = date_sub($dtrefind, date_interval_create_from_date_string('1 days'))->format('d/m/Y');
        }
         
        $produtosAFs = $this->lancamentoEntradaFactory->getProdutoAfImportadaIndice($nota['CDFILIAL'], $nota['NRLANCTONF']);

        foreach($produtosAFs as $produto){
           
            if( $produto['CDINDICE'] != null ){

                $cdindice = $produto['CDINDICE'];
                $cdtpindice = $produto['CDTPINDICE'];
                $idpervalor = $produto['IDPERVALOR'];
                $qtitemprod =  $this->lancamentoFactory->getQuantidadeProduto($nota['CDFILIAL'], $produto['NRLANCTONF'], $produto['CDPRODUTO'], $this->environment->getNrOrgTrab(), $produto['NRAUTOFORN']);
                $novoIndice =  $this->lancamentoEntradaFactory->getValorIndicePorDataEmissao($cdindice, $cdtpindice, $idpervalor, $dtrefind, $this->environment->getNrOrgTrab());

                $vrunititem = $produto['VRPREUNIAF'];
                
                $vrdescitem = ($vrunititem * $qtitemprod) * ($produto['VRPERDESAF'] / 100);
                $vrtotaitem = $vrunititem * $qtitemprod - (($vrunititem * $qtitemprod) * ($produto['VRPERDESAF'] / 100));
                    
                if ($novoIndice != null){
                    $vrunititem = $this->converterValorMoeda($novoIndice, $vrunititem);
                    $vrdescitem = $this->converterValorMoeda($novoIndice, $vrdescitem);
                    $vrtotaitem = $this->converterValorMoeda($novoIndice, $vrtotaitem);
                    $this->lancamentoFactory->updateValoresProdutosNF($produto['CDPRODUTO'], $nota['CDFILIAL'], $produto['NRLANCTONF'], $qtitemprod, $vrunititem, $vrdescitem, $vrtotaitem , $this->environment->getNrOrgTrab());
                }
                else{
                    
                    throw new \Exception('Não Foi possível realizar a conversão de moeda! Nao há valor do índice  para a data '.$dtrefind); 
                }

            }             

        }

    }
    
    public function consultaNotaUtilizaAfIndice($nota) {
        return $this->lancamentoEntradaFactory->consultaNotaUtilizaAfIndice($nota["CDFILIAL"], $nota["NRLANCTONF"]) > 0;
    }
    
    public function getDadosVenc($cdTipooper, $identrsaid, $cdForneced) {
        return $this->lancamentoEntradaFactory->getDadosVenc($cdTipooper, $identrsaid, $cdForneced);
    }
    
    public function gravaVencimento($rowLancamento, $dadosVenc) {
        $totalImpoRetido = $this->lancamentoEntradaFactory->getRetencoes($rowLancamento["CDFILIAL"], $rowLancamento["NRLANCTONF"]); 
        $totalBrutoVenc  = $this->lancamentoEntradaFactory->subtVrBruto($rowLancamento["CDFILIAL"], $rowLancamento["NRLANCTONF"]);
        $reTotaVenc = $rowLancamento["VRNOTAFISC"] - $totalImpoRetido - $totalBrutoVenc;
        $vencimentos = array();
        
        if($reTotaVenc > 0) {
            $afs = $this->lancamentoEntradaFactory->getContAf($rowLancamento["CDFILIAL"], $rowLancamento["NRLANCTONF"]);

            if(!empty($afs) && count($afs) == 1) {
                if($afs) {
                    $af = $afs[0];
                }
                $vencimentos = $this->vencimentoService->criaVencimentos('A', $rowLancamento["CDFORNECED"], $reTotaVenc, $rowLancamento["DTEMISSAO"], $rowLancamento["DTENTRSAID"], false, $rowLancamento["CDFILIRECE"], '', '', $af);
            }
            else {
                $vencimentos = $this->vencimentoService->criaVencimentos('E', $rowLancamento["CDFORNECED"], $reTotaVenc, $rowLancamento["DTEMISSAO"], $rowLancamento["DTENTRSAID"], false, $rowLancamento["CDFILIRECE"]);               
            }
        }
        
        if(!empty($vencimentos)) {
            $vencimentos = $this->preencheDadosVencimento($vencimentos, $rowLancamento);
            $this->vencimentoService->insereVencimentos('contapagar', $vencimentos);
        }
    }
    
    protected function preencheDadosVencimento($vencimentos, $nota) {
        $vencimentosAjustados = array();
        $count = 1;

        foreach($vencimentos as $vencimento) {
            $vencimentosAjustados[] = array(
                "CDFILIAL"      => $nota["CDFILIAL"],
                "NRLANCTONF"    => $nota["NRLANCTONF"],
                "NRSEQVENPAG"   => str_pad($count, 2, "0", STR_PAD_LEFT),
                "NRPARCELAPAG"  => "00",
                "QTDIASVENPAG"  => $vencimento["NRDIASPARCELA"],
                "DTORIGVENPAG"  => $vencimento["DTPARCELA"],
                "DTATUAVENPAG"  => $vencimento["DTPARCELA"],
                "VRORIGPAG"     => $vencimento["VRPARCELA"],
                "VRATUAPAG"     => $vencimento["VRPARCELA"],
                "__is_new"      => true
            );    
            $count ++;
        }
        return $vencimentosAjustados;
    }
    
    public function deletarAnexoFileServer($path){
        try {
            return $this->fileUploader->deleteFile($path);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    

   /******************************************* Salvar Anexo *********************************************/
    public function salvaAnexo($row){
        try {
            $anex = $row['DSLINKANEXO'];
            $anex = $anex[0];
            $nome = '<'.$anex['name'].'>'.$anex['path'];
            
            $nrseqAnexo = $this->lancamentoEntradaFactory->getSequencialAnexonfe($row["CDFILIAL"], $row["NRLANCTONF"])["NRSEQANEXO"];
            $nrseqAnexo = str_pad($nrseqAnexo,  4, "0", STR_PAD_LEFT);
            $row["NRSEQANEXO"] = $nrseqAnexo;
            $this->lancamentoEntradaFactory->salvaAnexo($nome,$this->environment->getCdOperador(),
                    $this->environment->getNrOrgTrab(),$row['NRLANCTONF'],$row['CDFILIAL'], $row["NRSEQANEXO"]);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /******************************************** Download Anexo ****************************************/
    public function baixaAnexo($row){
        $path = $this->fileService.$row['path'];
        return $path;
    }
    
    public function insereNotasOpOnze($rows, $lancamentoNotafiscalService) {
        try {
            $cabecalho = $rows[0];
            $itensRow = $rows[1];
            
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $nrorg = $this->environment->getNrOrgTrab();
            $filial = $itensRow[0]["CDFILIALN"];
            $lancamento = $itensRow[0]["NRLANCTONFN"];
            $cdfilirece = $itensRow[0]["CDFILIREC"];
            $nrlanctonfit = '_';

            foreach($itensRow as $row) {
                $this->lancamentoEntradaFactory->insereItemNotaFisc($filial, $lancamento, $row["CDFILIAL"], $row["NRLANCTONF"], $nrorg);
                $nrlanctonfit .= $row["NRLANCTONF"].'_';
            }
            
            //verifica se a nota foi gerada por xml, caso sim, os produtos que retornarão na tela de cabecalho->produtos serão os trazidos pelo xml. Caso não serão importados os itens da nota de origem.
            $isImportacaoXml = $this->lancamentoEntradaFactory->verificaNracessoOpOnze($filial, $lancamento, $nrorg);

            if($isImportacaoXml["IDNFGERADAVIAXML"] == 'S'){
                $connection->commit();
                return;
            }
            
            $itens = $this->lancamentoEntradaFactory->getItensEntradaSemNota($filial, $lancamento, $row["CDFILIAL"], $nrlanctonfit, $nrorg);
            
            $maxSeqItem =  $this->lancamentoEntradaFactory->getMaxSeqItem($filial, $lancamento);
            
            if(!$maxSeqItem) {
                $seqItem = 0;
            }
            else {
                $seqItem = $maxSeqItem + 1;
            }
            
            foreach($itens as $item) {
                $nrseqitem = str_pad($seqItem, 4, '0', STR_PAD_LEFT);
                
                $existeItemProduto =  $this->lancamentoEntradaFactory->existeItemProduto($filial, $lancamento, $item["CDPRODEMIS"], $item["VRUNITITEM"], 'S', $item["DSPRODEMIS"],
                                                            $item["VRDESCITEM"], 'S', $item["CDPRODUTO"], $item["QTITEMPROD"],
                                                            0, ($item["VRTOTAITEM"] - $item["VRDESCITEM"]), $cdfilirece, $item["CDALMOXARIFE"], $item["CDLOCALESTOQ"], $item["NRLOTEESTQ"], $item["NRSUBLOTE"], $this->environment->getCdOperador());
                $existeItemProduto["NRSEQITPROD"];
                if($existeItemProduto){ 
                    $item["VRDESCITEM"] = $item["VRDESCITEM"] + $existeItemProduto["VRDESCITEM"];
                    $item["VRTOTAITEM"] = $item["VRTOTAITEM"] + $existeItemProduto["VRTOTAITEM"];
                    $item["QTITEMPROD"] = $item["QTITEMPROD"] + $existeItemProduto["QTITEMPROD"];
                    $vrbrutitem         = ($item["VRTOTAITEM"] - $item["VRDESCITEM"]) +  ($existeItemProduto["VRTOTAITEM"] - $existeItemProduto["VRDESCITEM"]);
                    $this->lancamentoEntradaFactory->updateItemProduto($filial, $lancamento, $item["CDPRODEMIS"], $item["VRUNITITEM"], 'S', $item["DSPRODEMIS"],
                                                            $item["VRDESCITEM"],$existeItemProduto["NRSEQITPROD"], 'S', $item["VRTOTAITEM"], $item["CDPRODUTO"], $item["QTITEMPROD"],
                                                            0, $vrbrutitem, $cdfilirece, $item["CDALMOXARIFE"], $item["CDLOCALESTOQ"], $item["NRLOTEESTQ"], $item["NRSUBLOTE"], $this->environment->getCdOperador());
                    
                    $item['NRSEQITPROD'] = $existeItemProduto["NRSEQITPROD"];
                    $lancamentoNotafiscalService->calculaImpostoItemProduto($cabecalho,$item, false);
  
              
                }else{
                    $this->lancamentoEntradaFactory->insereItemProduto($filial, $lancamento, $item["CDPRODEMIS"], $item["VRUNITITEM"], 'S', $item["DSPRODEMIS"],
                                                            $item["VRDESCITEM"], $nrseqitem, 'S', $item["VRTOTAITEM"], $item["CDPRODUTO"], $item["QTITEMPROD"],
                                                            0, ($item["VRTOTAITEM"] - $item["VRDESCITEM"]), $cdfilirece, $item["CDALMOXARIFE"], $item["CDLOCALESTOQ"], $item["NRLOTEESTQ"], $item["NRSUBLOTE"], $this->environment->getCdOperador());
                    $item['NRSEQITPROD'] = $nrseqitem;
                    $lancamentoNotafiscalService->calculaImpostoItemProduto($cabecalho,$item, false);
                }
                
                
                $seqItem += 1;
               
            }

            $connection->commit();
            
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }

    }
    
    public function novoImpostoItemTemp($rows) {
        try{
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            foreach($rows as $row) {
                $this->lancamentoEntradaFactory->novoImpostoItemTemp(
                    $row['CDFILIMPOIT'],
                    $row['NRLANCIMPOIT'], 
                    $row['NRSEQIMPOIT'], 
                    $row['CDIMPOSTOIT'], 
                    $row['NRORG'], 
                    $row['CDOPERINCLUSAO'], 
                    $row['VRIMPOSIT'],
                    $row['VRALIQIMPOIT'], 
                    $row['VRBCALCIMPOIT'], 
                    $alteracaoUser = 'N', 
                    $row['NRSEQITPRO'] = 1, 
                    $row['VRCONTIMPOIT'], 
                    $row['VRREDBCIMPOIT'], 
                    $row['VRPDEDIMPOIT'], 
                    $row['IDIMPDESOTRABIT'],
                    $row['IDMOTDESOTRABIT'], 
                    $iddifaliqcuspti = 'N', 
                    $row['VRADICFECP'], 
                    $row['VRADICALIQFECP'], 
                    $row['IDVALORINFO'], 
                    $row['CDSTRIBIMPOIT']
                );
            } 
            $this->entityManager->flush();
            $connection->commit();
            
        }  catch(\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }
}