<?php
namespace Teknisa\DF\Services;

use Teknisa\Libs\Util\Environment;
use Teknisa\Model\Geral\Entities\Tipooperac;
use Teknisa\DF\Factories\Ger06600TipoDeOperacaoFactory;

class Ger06600TipoDeOperacaoService {

    protected $environment;
    protected $entityManager;
    protected $factory;
    protected $defaultNotasEntrada = array(
        "01" => "NF Compra",
        "02" => "NF Transferência",
        "03" => "NF Complemento",
        "04" => "NF Simples Remessa",
        "05" => "NF Cobrança",
        "06" => "NF Brindes/Doações (Cliente)",
        "07" => "NF Movimentação",
        "08" => "NF Produtor Rural",
        "09" => "Vale de Transferência",
        "10" => "Entrada sem Nota Fiscal",
        "11" => "NF Atrasada(Cobrança de ESNF)",
        "12" => "Recibo Pgto. a Autônomo(RPA)",
        "13" => "NF de Cliente",
        "16" => "NF de Abastecimento"
    );
    protected $defaultNotasSaida = array(
        "01" => "NF Venda",
        "02" => "NF Transferência",
        "03" => "NF Complemento",
        "04" => "NF Simples Remessa",
        "05" => "NF Cobrança",
        "06" => "NF Devolução ao Fornecedor",
        "07" => "NF Movimentação",
        "08" => "NF Outras Saídas",
        "09" => "Vale de Transferência",
        "10" => "Saída sem Nota Fiscal",
        "11" => "NF Cobrança (Saída sem NF)",
        "12" => "Nota de Débito/Crédito",
        "13" => "NF Prestação de Serviços",
        "14" => "NF de Armazenamento",
        "15" => "NF Retorno de Armazenamento",
        "16" => "NF de Abastecimento"
    );

    public function __construct(Environment $environment, \Doctrine\ORM\EntityManager $entityManager, Ger06600TipoDeOperacaoFactory $factory) {
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->factory = $factory;
    }
    
    public function insereTipoCusto(){
        $this->factory->insereTipoCusto();
    }

    public function getParameters($row) {
        $nrorg = $this->environment->getNrorgTrab();
        $tipooperac = $this->entityManager->getRepository(Tipooperac::class)->findOneBy(array(
            "identrsaidop" => $row['IDENTRSAIDOP'],
            "cdtipooper" => $row['CDTIPOOPER'],
            "nrorg" => $nrorg
        ));
        if ($tipooperac === null) {
            $parameters = array("IDGERIPIDESPA" => 'N', "IDGERICMSTDESPA" => 'N');
        }
        else {
            $parameters = array("IDGERIPIDESPA" => $tipooperac->getIdgeripidespa(), "IDGERICMSTDESPA" => $tipooperac->getIdgericmstdespa());
        }
        return $parameters;
    }

    public function validateDatabase($identrsaidop) {
        if ($identrsaidop !== 'S' && $identrsaidop !== 'E') {
            throw new \Exception("ID de Entrada/Saída inválido.");
        }
        $nrorg = $this->environment->getNrorgTrab();
        $operador = $this->environment->getCdOperador();
        $currentTime = $this->environment->getCurrentTime();
        $tipooperacRepository = $this->entityManager->getRepository(Tipooperac::class);
        $notasDefault = $identrsaidop === 'E' ? $this->defaultNotasEntrada : $this->defaultNotasSaida;

        foreach ($notasDefault as $cdtipooper => $nmtipooper) {
            $tipooperac = $tipooperacRepository->findOneBy(array(
                "identrsaidop" => $identrsaidop,
                "cdtipooper" => $cdtipooper,
                "nrorg" => $nrorg
            ));
            if ($tipooperac === null) {
                $tipooperac = new Tipooperac();
                $tipooperac->setNrorg($nrorg);
                $tipooperac->setIdentrsaidop($identrsaidop);
                $tipooperac->setCdtipooper($cdtipooper);
                $tipooperac->setNmtipooper($nmtipooper);
                $tipooperac->setIdorigdestop($identrsaidop === 'E' ? 'F' : 'C');
                $tipooperac->setIdserieauto('N');
                $tipooperac->setIdutilvencobr('N');
                $tipooperac->setIdconsparfis('N');
                $tipooperac->setIdgeravencnfap('N');
                $tipooperac->setIdutgernftran('N');
                $tipooperac->setIdinfcpfcnpjnf('N');
                $tipooperac->setCdfornecedpadr('');
                $tipooperac->setDsopernfavista('');
                $tipooperac->setIdutcusorigest('N');
                $tipooperac->setIdnotadevo('N');
                $tipooperac->setIdutilnfse('N');
                $tipooperac->setIdbrindetpoper('N');
                $tipooperac->setIdcoavrimpvrprod('N');
                $tipooperac->setIdnfprestserv('N');
                $tipooperac->setIdnfestcanc('N');
                $tipooperac->setIdcontaboper('S');
                $tipooperac->setIdnfajuste('N');
                $tipooperac->setIdutctrlaidf('N');
                $tipooperac->setIdnfimportacao('N');
                $tipooperac->setIdtipooperele('N');
                $tipooperac->setIdoperdevool('N');
                $tipooperac->setIdaltdtrefaprov('N');
                $tipooperac->setIdobrdtreffat('N');
                $tipooperac->setIdcontsomenimpo('N');
                $tipooperac->setIdnfretind('N');
                $tipooperac->setIdcontconforig('S');
                $tipooperac->setIddescimpreccofi('N');
                $tipooperac->setIdremessaindu('N');
                $tipooperac->setIdoptriangular('N');
                $tipooperac->setIdentrterceiro('N');
                $tipooperac->setIdcmpvndboni('N');
                $tipooperac->setIdativo('S');
                $tipooperac->setDtinclusao($currentTime);
                $tipooperac->setCdoperinclusao($operador);
                $tipooperac->setNrorginclusao($nrorg);
                $this->entityManager->persist($tipooperac);
                $this->entityManager->flush();
            }
        }
    }

    public function validateRowToSave($row) {
        if (empty($row["IDENTRSAIDOP"])) {
            throw new \Exception("Identificador Entrada/Saída não informado.");
        }
        if ($row["IDENTRSAIDOP"] != 'S' && $row["IDENTRSAIDOP"] != 'E') {
            throw new \Exception("Identificador Entrada/Saída inválido.");
        }
        if (empty($row["CDTIPOOPER"])) {
            throw new \Exception("Código não informado.");
        }
        if (empty($row["NMTIPOOPER"])) {
            throw new \Exception("Nome não informado.");
        }
        if ($row['__is_new']) {
            $nrorg = $this->environment->getNrorgTrab();
            $tipooperac = $this->entityManager->getRepository(Tipooperac::class)->findOneBy(array(
                "identrsaidop" => $row['IDENTRSAIDOP'],
                "cdtipooper" => $row['CDTIPOOPER'],
                "nrorg" => $nrorg
            ));
            if ($tipooperac !== null) {
                throw new \Exception("Tipo de Operação já cadastrado.");
            }
        }
        if ($row["IDNFESTCANC"] == 'S' && $row["IDOPERDEVONF"] != 'S') {
            throw new \Exception("Atenção! Tipo de operação de estorno de cancelamento deve utilizar documento de origem.");
        }
        if ($row["IDNFESTCANC"] == 'S' && $row['IDNFORIGOBRI'] != 'S') {
            throw new \Exception("Operação bloqueada! Tipo de operação de estorno de cancelamento deve utilizar documento de origem obrigatório.");
        }
        if ($row["IDENTRSAIDOP"] == 'E' && $row['CDTIPOOPER'] == '13') {
            if ($row["IDOPERDEVONF"] != 'S') {
                throw new \Exception("Operação bloqueada! Este tipo de operação deve utilizar documento de origem.");
            }
            if ($row["IDNFORIGOBRI"] == 'N') {
                throw new \Exception("Operação bloqueada! Este tipo de operação não pode utilizar documento de origem Não obrigatório.");
            }
           
        }
        if ($row["IDENTRSAIDOP"] == 'S' && $row['CDTIPOOPER'] == '06') {
            if ($row["IDOPERDEVONF"] != 'S') {
                throw new \Exception("Operação bloqueada! Este tipo de operação deve utilizar documento de origem.");
            }
            if ($row["IDNFORIGOBRI"] != 'S') {
                throw new \Exception("Operação bloqueada! Este tipo de operação deve utilizar documento de origem obrigatório.");
            }
        }
        if ($row['CDTIPOOPER'] == '05' || $row['CDTIPOOPER'] == '11') {
            if ($row["IDOPERDEVONF"] == 'S') {
                throw new \Exception("Operação bloqueada! Este tipo de operação não pode atualizar estoque.");
            }
        }
        if ($row['IDOPERDEVONF'] != 'S') {
            $parameters = $this->getParameters($row);
            if ($parameters['IDGERIPIDESPA'] == 'S' || $parameters['IDGERICMSTDESPA'] == 'S') {
                throw new \Exception("Operação bloqueada! Existe parametrização de despesa acessória. Favor verificar controle fiscal.");
            }
            if ($row["IDDCORIGRELAC"] == 'S') {
                throw new \Exception("Operação bloqueada! Este tipo de operação não pode utilizar 'Documento de Origem não poderá ser relacionado a mais de uma NF'");
            }
            if ($row["IDNFORIGOBRI"] == 'S') {
                throw new \Exception("Operação bloqueada! Este tipo de operação não pode utilizar 'Documento Origem Obrigatório'");
            }
            if ($row["IDCONTCONFORIG"] == 'S') {
                throw new \Exception("Operação bloqueada! Este tipo de operação não pode utilizar 'Contabiliza conforme NF de origem'");
            }
            if ($row["IDUTLLISTORI"] == 'S') {
                throw new \Exception("Operação bloqueada! Este tipo de operação não pode utilizar 'Utiliza Pesquisa em Lista'");
            }
            if ($row["DSOPERNFORIG"] != null) {
                throw new \Exception("Operação bloqueada! Este tipo de operação não pode utilizar 'Documento Origem'");
            }
        }
        else {
            if ($row["IDDCORIGRELAC"] == 'S' && $row["IDENTRSAIDOP"] == 'S') {
                throw new \Exception("Operação bloqueada! Um tipo de operação de saída não pode ter o documento de origem relacionado a mais de uma NF.");
            }
            if ($row["IDUTLLISTORI"] == 'S' && $row["IDNFESTCANC"] == 'S') {
                throw new \Exception("Operação bloqueada! Este tipo de operação não pode utilizar pesquisa em lista se for parametrizado para utilizar NF de estorno de cancelamento.");
            }
        }
        if ($row['__is_new'] && $row["IDNFESTCANC"] == $row["IDCONTABOPER"]) {
            throw new \Exception("Erro! Ao inserir um tipo de operação, o parâmetro \"NF de Estorno de Cancelamento.\" deve ser diferente do parâmetro \"IDCONTABOPER\"");
        }
    }
}
